/*
 * Created on 28.02.2004
 */
package org.n52.wmsclientcore;

import java.awt.image.renderable.*;
import java.io.*;
import java.math.*;
import java.net.*;
import java.util.*;

import javax.media.jai.*;

import org.apache.log4j.*;
import org.exolab.castor.xml.*;
import org.n52.wmsclientcore.context.*;
import org.n52.wmsclientcore.operations.*;
import org.n52.wmsclientcore.request.*;
import org.n52.wmsclientcore.util.*;
import org.xml.sax.*;

import com.sun.media.jai.codec.*;


/**
 * This is the central class for controlling the complete framework. It contains the complete model and has some
 * common methods to do some common stuff (getMap, 
 * @author Theodor
 */
public class ClientModelController {
    protected ViewContext context;

    protected WmsCollection collection;

    protected ImageOverlay imageOverlay;

    protected static Logger logger = Logger.getLogger(ClientModelController.class
            .getName());

    protected int currentFitStrategy = WmcBoundingBox.FIT_MINIMUM;

    /*
     * Initializing the controller by a simple url, which points to a
     * contextfile.
     */
    public ClientModelController(String url) throws WmsException {
        try {
            SniffedXMLInputStream sniffedInputStream = new SniffedXMLInputStream(new URL(url).openStream());
            String encoding = sniffedInputStream.getXmlEncoding();
            InputStreamReader reader = new InputStreamReader(sniffedInputStream, encoding);
            context = (ViewContext) ViewContext
                    .unmarshal(reader);
            collection = new WmsCollection();
            synchronizeWmsCollection();
            this.imageOverlay = new ImageOverlay();

        } catch (MarshalException e) {
            throw new WmsException(e);
        } catch (ValidationException e) {
            throw new WmsException(e);
        } catch (MalformedURLException e) {
            throw new WmsException(e);
        } catch (IOException e) {
            throw new WmsException(e);
        }
    }
    /**
     * Initializes a controller with a ViewContext Object.
     * @param ctx
     * @throws WmsException
     */
    public ClientModelController(ViewContext ctx) throws WmsException {
        context = ctx;
        collection = new WmsCollection();
        synchronizeWmsCollection();
        this.imageOverlay = new ImageOverlay();
    }
    
    /**
     * This is the standard constructor and initializes the context by using the WmsResources and the 
     * Context which is defined default in the properties file or in the web.xml.
     *  
     * @throws WmsException
     */
    public ClientModelController() throws WmsException {
        this(WmsResources.getInitialContext());
    }
    
    public ClientModelController(InputStream inputStream, String encoding)
            throws WmsException {
        try {
            

            InputStreamReader isr = new InputStreamReader(inputStream,
                    encoding);
            context = (ViewContext) ViewContext.unmarshal(isr);

            collection = new WmsCollection();
            synchronizeWmsCollection();
            this.imageOverlay = new ImageOverlay();
        } catch (IOException e) {
            throw new WmsException(e);
        } catch (MarshalException e) {
            throw new WmsException(e);
        } catch (ValidationException e) {
            throw new WmsException(e);
        }

    }
    /*
     * This inserts a Layer Element into the context. It adds also styles...
     * Please check for double added Layers!
     */
    public void insertLayerInModel(String wmsUrl, String chosenLayer) {
        org.n52.wmsclientcore.capabilities.model.Layer capsLayer = getCollection()
                .getWebMapService(wmsUrl).getMapLayerByName(chosenLayer);
        WebMapService service = getCollection().getWebMapService(wmsUrl);
        String[] formats = service.getCapabilities().getCapability()
                .getRequest().getGetMap().getFormat();
        FormatList formatList = new FormatList();
        //PERHAPS THE FORMATS CAN BE NULL... THIS HAS TO BE CHECKED.

        for (int i = 0; i < formats.length; i++) {
            Format format = new Format();
            format.setContent(formats[i]);
            formatList.addFormat(format);
        }
        org.n52.wmsclientcore.context.Layer contextLayer = new org.n52.wmsclientcore.context.Layer();
        contextLayer.setFormatList(formatList);
        contextLayer.setHidden(true);
        Server server = new Server();
        OnlineResource onlineRes = new OnlineResource();
        onlineRes.setHref(wmsUrl);
        server.setOnlineResource(onlineRes);
        server.setService("OGC:WMS");
        String serviceVersion = getCollection().getWebMapService(wmsUrl)
                .getVersion();
        server.setVersion(serviceVersion);
        if (capsLayer.getStyle() != null && capsLayer.getStyleCount() > 0) {
            StyleList styleList = new StyleList();
            for (int i = 0; i < capsLayer.getStyleCount(); i++) {
                Style style = new Style();
                
                style.setStyleTypeSequence(new StyleTypeSequence());
                style.getStyleTypeSequence().setName(
                        capsLayer.getStyle(i).getName());
                style.getStyleTypeSequence().setTitle(
                        capsLayer.getStyle(i).getTitle());
                if(capsLayer.getStyle(i).getLegendURL() != null && capsLayer.getStyle(i).getLegendURL().length > 0)
                {
                    LegendURL ctxLegendURL = new LegendURL();
                    OnlineResource or = new OnlineResource();
                    or.setHref(capsLayer.getStyle(i).getLegendURL(0).getOnlineResource().getHref());
                    ctxLegendURL.setOnlineResource(or);
                    ctxLegendURL.setFormat(capsLayer.getStyle(i).getLegendURL(0).getFormat());
                    ctxLegendURL.setHeight(capsLayer.getStyle(i).getLegendURL(0).getHeight());
                    ctxLegendURL.setWidth(capsLayer.getStyle(i).getLegendURL(0).getWidth());
                    style.getStyleTypeSequence().setLegendURL(ctxLegendURL);
                }
                styleList.addStyle(style);
            }
            styleList.getStyle(0).setCurrent(true);
            contextLayer.setStyleList(styleList);
        }
        contextLayer.setQueryable(true);
        contextLayer.setServer(server);
        contextLayer.setName(capsLayer.getName());
        contextLayer.setTitle(capsLayer.getTitle());
        context.getLayerList().appendLayer(contextLayer);

    }



    /**
     * @return
     */
    public WmsCollection getCollection() {
        return collection;
    }

    /**
     * @return
     */
    public ViewContext getContext() {
        return context;
    }

    public void setContext(ViewContext vc) {
        context = vc;
    }

    public LayerList getVisibleLayers() {
        LayerList returnLayerList = new LayerList();
        LayerList layerList = context.getLayerList();
        for (int i = 0; i < layerList.getLayerCount(); i++) {
            if (!layerList.getLayer(i).getHidden())
                returnLayerList.addLayer(layerList.getLayer(i));
        }
        return returnLayerList;
    }

    private LayerList getVisibleLayersAsLayerList() {
        LayerList returnList = new LayerList();
        LayerList layerList = context.getLayerList();
        for (int i = 0; i < layerList.getLayerCount(); i++) {
            if (!layerList.getLayer(i).getHidden())
                returnList.addLayer(layerList.getLayer(i));
        }
        return returnList;
    }

    public void addWms(String url, String version) throws WmsException {
        try {
            collection.addWebMapService(url, version);
        } catch (Exception e) {
            collection.remove(url);
            throw new WmsException("wms not added, invalid URL");
        }
    }

    public void addWms(String url) throws WmsException {
        collection.addWebMapService(url);
    }

    /*
     * This method synchronizes the collection with the actual context. This
     * happens, if a new context is added, or for the initial Controller
     */
    protected void synchronizeWmsCollection() throws WmsException {
        LayerList list = context.getLayerList();
        for (int i = 0; i < list.getLayerCount(); i++) {
            try {
                String serverURL = list.getLayer(i).getServer()
                        .getOnlineResource().getHref();

                if (!collection.containsKey(serverURL)) {
                    collection.addWebMapService(serverURL, list.getLayer(i)
                            .getServer().getVersion());
                    //WebMapService service =
                    // collection.getWebMapService(serverURL);
                }
                if (list.getLayer(i).getStyleList() != null) {
                    if (list.getLayer(i).getStyleList().getStyle(0).getSLD() != null) {
                        String sldUrl = list.getLayer(i).getStyleList()
                                .getStyle(0).getSLD().getSLDTypeChoice()
                                .getOnlineResource().getHref();
                        
                            try {
                                ((WebMapService) collection
                                        .getWebMapService(serverURL))
                                        .addSldUrl(sldUrl);
                            } catch (IOException e) {
                                throw new WmsException(e);
                            } catch (SAXException e) {
                                throw new WmsException(e);
                            }
                        
                    }
                }
            } catch (IndexOutOfBoundsException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }
    }

    public void putLayerOnTop(String layerName, String wmsUrl) {
        LayerList layerList = context.getLayerList();
        LayerList newLayerList = new LayerList();
        Layer layerOnTop = null;
        for (int i = 0; i < layerList.getLayerCount(); i++) {
            if (layerList.getLayer(i).getName().equals(layerName)
                    && layerList.getLayer(i).getServer().getOnlineResource()
                            .getHref().equals(wmsUrl))
                layerOnTop = layerList.getLayer(i);
            else
                newLayerList.addLayer(layerList.getLayer(i));
        }
        if (layerOnTop == null) {
            //do nothing!!!
        } else
            newLayerList.addLayer(layerOnTop);
        context.setLayerList(newLayerList);
    }

    public void moveLayerUp(String layerName, String wmsUrl) {
        LayerList layerList = context.getLayerList();
        LayerList newLayerList = new LayerList();
        for (int i = 0; i < layerList.getLayerCount(); i++) {
            if (layerList.getLayer(i).getName().equals(layerName)
                    && i < layerList.getLayerCount() - 1
                    && layerList.getLayerCount() > 1) {
                newLayerList.addLayer(layerList.getLayer(i + 1));
                newLayerList.addLayer(layerList.getLayer(i));
                i++;
            } else
                newLayerList.addLayer(layerList.getLayer(i));

        }
        context.setLayerList(newLayerList);
    }

    /*
     * ACHTUNG DIESE METHODE IST SAU INPERFORMANT WEGEN APPENDLAYER() ;)
     */
    public void moveLayerDown(String layerName, String wmsUrl) {
        LayerList layerList = context.getLayerList();
        LayerList newLayerList = new LayerList();
        for (int i = layerList.getLayerCount() - 1; i >= 0; i--) {
            if (layerList.getLayer(i).getName().equals(layerName) && i > 0
                    && layerList.getLayerCount() > 1) {
                newLayerList.appendLayer(layerList.getLayer(i - 1));
                newLayerList.appendLayer(layerList.getLayer(i));
                i--;
            } else
                newLayerList.appendLayer(layerList.getLayer(i));

        }
        context.setLayerList(newLayerList);
    }

    /**
     * GetMap will be called for each WMS stored by the collection.
     * 
     * @return PlanarImage
     */
    public PlanarImage getMap() {

        // clear image list
        imageOverlay.clearImageList();
        LayerList visibleLayers = this.getVisibleLayersAsLayerList();
        if (visibleLayers.getLayerCount() != 0) {
            String currentWmsUrl = new String();
            GetMapOperation op = null;
            GetMapRequest req = null;
            String tempWmsID = new String("");
            WebMapService wms = null;
            for (int i = visibleLayers.getLayerCount() - 1; i >= 0; i--) {
                //boolean stillSequence = true;
                String wmsIDOfCurrentLayer = (visibleLayers.getLayer(i))
                        .getServer().getOnlineResource().getHref();
                wms = collection.getWebMapService(wmsIDOfCurrentLayer);
                if (!wmsIDOfCurrentLayer.equals(tempWmsID)
                        && !tempWmsID.equals(""))
                    imageOverlay.addImage(this.retrieveMap(req, wms
                            .getOnlineRessource()));

                if (sldStyleActivated(visibleLayers.getLayer(i))) {
                    op = new GetMapOperation(wms.getCapabilities());
                    req = op.getGetMapRequest();

                    setGeneralValues(req);
//                    for (int j = 0; j < visibleLayers.getLayer(i)
//                            .getStyleList().getStyleCount(); j++) {
//                        if (visibleLayers.getLayer(i).getStyleList()
//                                .getStyle(j).getCurrent())
//                            req.setSld(visibleLayers.getLayer(i).getStyleList()
//                                    .getStyle(j).getSLD().getSLDTypeChoice()
//                                    .getOnlineResource().getHref());
//                    }
                    //req.setSld();

                    //req.setLayers("");
                    //req.setStyles("");
                    req.setTransparent(true);
                    //setGeneralValues(req);
                    tempWmsID = wmsIDOfCurrentLayer;
                } else if (!wmsIDOfCurrentLayer.equals(tempWmsID)
                        || tempWmsID.equals("")) {
                    //		Opening the new FeatureInfoRequest
                    tempWmsID = wmsIDOfCurrentLayer;
                    op = new GetMapOperation(wms.getCapabilities());
                    req = op.getGetMapRequest();

                    setGeneralValues(req);

                }
                //if (!sldStyleActivated(visibleLayers.getLayer(i)))
                    setMapRequestForLayer(req, visibleLayers.getLayer(i), wms
                            .getMapLayerByName(visibleLayers.getLayer(i)
                                    .getName()));
            }
            imageOverlay.addImage(this.retrieveMap(req, wms
                    .getOnlineRessource()));
        }


        return imageOverlay.doOverlay();
    }

    private boolean sldStyleActivated(Layer layer) {
        if (layer.getStyleList() != null) {
            if (layer.getStyleList().getStyleCount() > 0) {
                for (int i = 0; i < layer.getStyleList().getStyleCount(); i++) {
                    if (layer.getStyleList().getStyle(i).getCurrent()
                            && layer.getStyleList().getStyle(i).getSLD() != null)
                        return true;

                }

            }
        }
        return false;
    }

    private void setGeneralValues(GetMapRequest req) {
        WmcBoundingBox bBox = new org.n52.wmsclientcore.WmcBoundingBox(context
                .getGeneral().getBoundingBox());
        bBox.fitBBox2Screen(context.getGeneral().getWindow().getWidth(),
                context.getGeneral().getWindow().getHeight(),
                currentFitStrategy);
        RequestBoundingBox reqBox = new RequestBoundingBox(bBox.getSRS());
        reqBox.setMaxx(bBox.getUpperRight().getX());
        reqBox.setMaxy(bBox.getUpperRight().getY());
        reqBox.setMinx(bBox.getLowerLeft().getX());
        reqBox.setMiny(bBox.getLowerLeft().getY());
        req.setBbox(reqBox);
        context.getGeneral().getBoundingBox().setMinx(
                new BigDecimal(bBox.getLowerLeft().getX()));
        context.getGeneral().getBoundingBox().setMaxx(
                new BigDecimal(bBox.getUpperRight().getX()));
        context.getGeneral().getBoundingBox().setMiny(
                new BigDecimal(bBox.getLowerLeft().getY()));
        context.getGeneral().getBoundingBox().setMaxy(
                new BigDecimal(bBox.getUpperRight().getY()));
        req.setCrs(context.getGeneral().getBoundingBox().getSRS());
        req.setHeight(Integer.toString(context.getGeneral().getWindow()
                .getHeight()));
        req.setWidth(Integer.toString(context.getGeneral().getWindow()
                .getWidth()));
        req.setFormat(WmsResources.getPropertyValue("imageFormat"));

    }

    /**
     * Returns a PlanerImage that stores the map specified by RequestObject
     * 
     * @param obj
     * @return PlanarImage
     */
    private PlanarImage retrieveMap(GetMapRequest obj, String wmsID) {
        String fullRequest = obj.getCompleteRequestURL();

        //fullRequest = fullRequest.replaceAll(" ", "%20");
        // handle scale hint
        String layerName = obj.getLayers();
        //double topLevelMaxScaleDenominator =
        // capabilities.getCapabilities().getCapability().getLayer().getMaxScaleDenominator();
        //uble topLevelMinScaleDenominator =
        // capabilities.getCapabilities().getCapability().getLayer().getMinScaleDenominator();
        org.n52.wmsclientcore.capabilities.model.Layer layer = null;
        try {
            layer = collection.getWebMapService(wmsID).getMapLayerByName(
                    layerName);

        } catch (Exception e) {
            System.err.println(e.getMessage());
        }

        if (layer != null) {
            double localMaxScale = layer.getMaxScaleDenominator();
            double localMinScale = layer.getMinScaleDenominator();
            if (localMaxScale != 0 || localMinScale != 0) {
                int height = this.getContext().getGeneral().getWindow()
                        .getHeight();
                int width = this.getContext().getGeneral().getWindow()
                        .getWidth();
                WmcBoundingBox bbox = new WmcBoundingBox(this.getContext()
                        .getGeneral().getBoundingBox());
                bbox.fitBBox2Screen(width, height, currentFitStrategy);
                double hint = bbox.currentScaleHint(width, height);
                if (localMaxScale < hint || localMinScale > hint) {
                    // ausserhalb der scalehint
                    logger.info("Request ausserhalb der scale hint");
                    return errorImage();
                }
            }
        }

        PlanarImage image = null;
        logger.debug("Request URL:" + fullRequest);

        if (ImageCache.getInstance().containsKey(fullRequest)
                && ImageCache.cachingEnabled()) {
            logger.info("getting image from cache: " + fullRequest);
            image = (PlanarImage) ImageCache.getInstance().get(fullRequest);
        } else {
            logger.info("getting image from server: " + fullRequest);

            ImageBuilder builder = new ImageBuilder();
            try {
                builder.doRequest(fullRequest);
            } catch (WmsException e) {
                logger.error(e);
                return errorImage();
            }
            image = builder.getImage();
            if (ImageCache.cachingEnabled())
                ImageCache.set(fullRequest, image);
        }
        return image;
    }

    /**
     * 
     * @return PlanarImage - errorImage
     */
    private PlanarImage errorImage() {

        InputStream errorImage = null;
        errorImage = this.getClass().getResourceAsStream(
                WmsResources.getPropertyValue("errorImageBlank"));

        // Create the ParameterBlock.
        ParameterBlock pb = new ParameterBlock();
        SeekableStream s = SeekableStream.wrapInputStream(errorImage, false);

        // Create the ParameterBlock and add the SeekableStream to it.
        pb.add(s);

        // Create the PNG operation.
        RenderedOp op = JAI.create("PNG", pb);
        PlanarImage image = op.createInstance();

        return image;

    }

    /*
     * checks if the given Layer is still in the context
     */
    public boolean containsLayerInContext(String wmsUrl, String layerName) {

        LayerList layerList = context.getLayerList();
        for (int i = 0; i < layerList.getLayerCount(); i++) {
            if (layerList.getLayer(i).getName().equals(layerName)
                    && layerList.getLayer(i).getServer().getOnlineResource()
                            .getHref().equals(wmsUrl))
                return true;
        }
        return false;
    }

    public Layer findLayerInContext(String wmsUrl, String layerName) {
        LayerList layerList = context.getLayerList();
        for (int i = 0; i < layerList.getLayerCount(); i++) {
            if (layerList.getLayer(i).getName().equals(layerName)
                    && layerList.getLayer(i).getServer().getOnlineResource()
                            .getHref().equals(wmsUrl))
                return layerList.getLayer(i);
        }
        return null;
    }

    public void setLayerAndStyle(String wmsURL, String layerName,
            String styleName) {
        LayerList layerList = context.getLayerList();
        for (int i = 0; i < layerList.getLayerCount(); i++) {
            if (layerList.getLayer(i).getName().equals(layerName)) {
                if (layerList.getLayer(i).getStyleList() != null) {
                    StyleList styleList = layerList.getLayer(i).getStyleList();
                    for (int j = 0; j < styleList.getStyleCount(); j++)
                        styleList.getStyle(j).setCurrent(false);
                    for (int j = 0; j < styleList.getStyleCount(); j++) {
                        if (styleList.getStyle(j).getStyleTypeSequence()
                                .getName().equals(styleName))
                            styleList.getStyle(j).setCurrent(true);
                    }
                }

            }
        }
    }

    private LayerList getVisibleAndQueryableLayersOfCtx() {
        LayerList returnList = new LayerList();
        for (int i = 0; i < context.getLayerList().getLayerCount(); i++) {
            if (!context.getLayerList().getLayer(i).getHidden()
                    && context.getLayerList().getLayer(i).getQueryable())
                returnList.addLayer(context.getLayerList().getLayer(i));
        }
        return returnList;
    }

    /**
     * requests feautureInfo for the specified layer. this method equals the
     * second getFeatureInfo method but requests only featureInfo for exactly
     * one layer.
     * 
     * @param layer
     * @param encoding
     * @param x
     * @param y
     * @return
     */
    public String getFeatureInfo(Layer layer, String encoding, int x, int y) {
        LayerList layerList = getVisibleAndQueryableLayersOfCtx();
        String wmsURL = layer.getServer().getOnlineResource().getHref();
        WebMapService wms = collection.getWebMapService(wmsURL);
        GetFeatureInfoOperation getFeatureInfoOperation = new GetFeatureInfoOperation(
                wms.getCapabilities());
        GetFeatureInfoRequest getFeatureInfoRequest = getFeatureInfoOperation
                .getFeatureInfoRequest();
        GetMapOperation getMapOperation = new GetMapOperation(wms
                .getCapabilities());
        GetMapRequest getMapRequest = getMapOperation.getGetMapRequest();
        this.setGeneralValues(getMapRequest);
        getFeatureInfoRequest.insertLayerBefore(layer.getName());
        this.setMapRequestForLayer(getMapRequest, layer, wms
                .getMapLayerByName(layer.getName()));
        getFeatureInfoRequest.setPartialMapRequest(getMapRequest
                .getPartialRequest());
        this.setFeatureInfoRequest(getFeatureInfoRequest, x, y, encoding);
        return this.retrieveFeatureInfo(getFeatureInfoRequest);
    }

    /**
     * generates a map with layerName as key and a string[] with all supported
     * FeatureInfoFormats as value.
     * 
     * @return
     */
    public Map getLayerNameToFeatureInfoFormatMapping() {
        Map result = new HashMap();
        LayerList layerList = getVisibleAndQueryableLayersOfCtx();
        String wmsURL;
        WebMapService wms;
        GetFeatureInfoOperation getFeatureInfoOperation;
        for (int i = 0; i < layerList.getLayerCount(); i++) {
            wmsURL = layerList.getLayer(i).getServer().getOnlineResource()
                    .getHref();
            wms = collection.getWebMapService(wmsURL);
            getFeatureInfoOperation = new GetFeatureInfoOperation(wms
                    .getCapabilities());
            result.put(layerList.getLayer(i).getName(), getFeatureInfoOperation
                    .getFormatsFromCapabilities());
        }
        return result;
    }

    /**
     * generates a map with layerName as key and Layer(ctx) as value for all
     * layers that are visible and queryable.
     * 
     * @return
     */
    public Map getVisibleAndQueryableLayers() {
        Map result = new LinkedHashMap();
        LayerList layerList = getVisibleAndQueryableLayersOfCtx();
        for (int i = 0; i < layerList.getLayerCount(); i++) {
            result.put(layerList.getLayer(i).getName(), layerList.getLayer(i));
        }
        return result;
    }

    public ArrayList getFeatureInfo(int x, int y, String encoding) {
        LayerList layerList = getVisibleAndQueryableLayersOfCtx();
        String tempWmsID = new String("");
        WebMapService wms = null;
        ArrayList featureInfo = new ArrayList();
        GetFeatureInfoOperation op = null;
        GetFeatureInfoRequest req = null;
        GetMapOperation mapOp = null;
        GetMapRequest mapReq = null;
        for (int i = layerList.getLayerCount() - 1; i >= 0; i--) {
            boolean stillSequence = true;
            String wmsIDOfCurrentLayer = (layerList.getLayer(i)).getServer()
                    .getOnlineResource().getHref();
            wms = collection.getWebMapService(wmsIDOfCurrentLayer);
            if (!wmsIDOfCurrentLayer.equals(tempWmsID) && !tempWmsID.equals("")) {
                //First closing the old FeatureInfoRequest
                req.setPartialMapRequest(mapReq.getPartialRequest());
                setFeatureInfoRequest(req, x, y, encoding);
                featureInfo.add(retrieveFeatureInfo(req));

            }
            if (!wmsIDOfCurrentLayer.equals(tempWmsID) || tempWmsID.equals("")) {
                //		Opening the new FeatureInfoRequest
                tempWmsID = wmsIDOfCurrentLayer;
                op = new GetFeatureInfoOperation(wms.getCapabilities());
                req = op.getFeatureInfoRequest();
                mapOp = new GetMapOperation(wms.getCapabilities());
                mapReq = mapOp.getGetMapRequest();
                setGeneralValues(mapReq);
            }
            req.insertLayerBefore(layerList.getLayer(i).getName());
            setMapRequestForLayer(mapReq, layerList.getLayer(i), wms
                    .getMapLayerByName(layerList.getLayer(i).getName()));
        }
        req.setPartialMapRequest(mapReq.getPartialRequest());
        setFeatureInfoRequest(req, x, y, encoding);
        featureInfo.add(retrieveFeatureInfo(req));
        return featureInfo;
    }

    private void setFeatureInfoRequest(GetFeatureInfoRequest req, int x, int y,
            String enc) {

        req.setInfoFormat(enc);
        req.setY(String.valueOf(y));
        req.setX(String.valueOf(x));
    }

    /*
     * establishes the connection to the Server and retrieves the FeatureInfo
     */
    private String retrieveFeatureInfo(GetFeatureInfoRequest req) {
        StringBuffer sb = new StringBuffer();
        try {
            URL url = new URL(req.getCompleteRequestURL());
            InputStreamReader reader = new InputStreamReader(url.openStream());
            BufferedReader br = new BufferedReader(reader);
            String line = null;
            while ((line = br.readLine()) != null)
                sb.append(line);
        } catch (IOException e1) {
            // TODO Auto-generated catch block
            e1.printStackTrace();
        }

        return sb.toString();
    }

    private void setMapRequestForLayer(GetMapRequest req, Layer ctxLayer,
            org.n52.wmsclientcore.capabilities.model.Layer capsLayer) {
        
        
        StyleList styles = ctxLayer.getStyleList();
        // capsLayer is null, if the Layer is defined through an SLD (as a UserLayer). 
        if(capsLayer != null) {
            req.setTransparent(!capsLayer.getOpaque());
            req.insertLayerBefore(ctxLayer.getName());
        }
        if (ctxLayer.getDimensionList() != null)
            for (int i = 0; i < ctxLayer.getDimensionList().getDimensionCount(); i++) {
                Dimension dim = ctxLayer.getDimensionList().getDimension(i);
                if (dim.getName().equals("time"))
                    req.setTime(dim.getContent());
            }
        String styleName = new String("");
        if (styles != null) {
            for (int j = 0; j < styles.getStyleCount(); j++) {
                if (styles.getStyle(j).getCurrent())
                    if(styles.getStyle(j).getSLD() != null){
                        req.setSld(styles.getStyle(j).getSLD().getSLDTypeChoice().getOnlineResource().getHref());
                    }
                    else {
                    styleName = styles.getStyle(j).getStyleTypeSequence()
                            .getName();
                    }
            }
        }
        req.insertStyleBefore(styleName);
    }
}