/*
 * Created on 09.06.2004
 */
package org.n52.wmsclientcore;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.math.BigDecimal;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import javax.media.jai.PlanarImage;
import javax.media.jai.TiledImage;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.n52.wmsclientcore.collection.ViewContextCollection;
import org.n52.wmsclientcore.context.BoundingBox;

import com.sun.media.jai.codec.ImageCodec;
import com.sun.media.jai.codec.ImageEncoder;
import com.sun.media.jai.codec.PNGEncodeParam;


/**
 * @author Theodor
 */
public class GeneralMapServlet extends HttpServlet {
	private Map generalMapControllerCollection;
	private String initialCtxID;
	/* (non-Javadoc)
	 * @see javax.servlet.http.HttpServlet#doGet(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
	 * this Servlet looks only for the parameter ctxid.
	 */

	protected void doGet(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException {
		// TODO Auto-generated method stub
		res.setContentType(WmsResources.getPropertyValue("image/png"));
		String ctxID = req.getParameter("ctxid");
		String currentCtxID = (String) req.getSession().getAttribute("currentCtxID");
		if (currentCtxID == null) {
			currentCtxID = initialCtxID;
			req.getSession().setAttribute("currentCtxID", currentCtxID);
		}
		if (ctxID != null) {
			currentCtxID = ctxID;
			req.getSession().setAttribute("currentCtxID", currentCtxID);
		}

		ClientModelController controller = (ClientModelController) req.getSession().getAttribute("controller");
		ClientModelController generalMapController = (ClientModelController) generalMapControllerCollection.get(currentCtxID);
		PlanarImage image = generalMapController.getMap();
		PNGEncodeParam ep = PNGEncodeParam.getDefaultEncodeParam(image);
		ep.setInterlacing(false);
		OutputStream out = res.getOutputStream();
		ImageEncoder ie = ImageCodec.createImageEncoder("PNG", out, ep);
		TiledImage tile = new TiledImage(image, true);
		Graphics2D graphics = tile.createGraphics();
		graphics.setPaint(Color.BLUE);
		graphics.setStroke(new BasicStroke(3));
		graphics.draw(calculateRect(controller, generalMapController));
		ie.encode(tile);

	}

	/* (non-Javadoc)
	 * @see javax.servlet.GenericServlet#init()
	 */
	public void init() throws ServletException {
		/*
		 * setting up the collection
		 */
		try {
			URL url = new URL(WmsResources.getPropertyValue("generalMapCollectionUrl"));

			ViewContextCollection coll = (ViewContextCollection) ViewContextCollection.unmarshal(new InputStreamReader(url.openStream()));
			generalMapControllerCollection = new HashMap();
			for (int i = 0; i < coll.getViewContextReferenceCount(); i++) {
				generalMapControllerCollection.put(coll.getViewContextReference(i).getId(), new ClientModelController(coll.getViewContextReference(i).getContextURL().getOnlineResource().getHref()));

			}
			//generalMapController = new ClientModelController();

		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		/*
		 * setting up the id for the initialctx
		 */
		initialCtxID = WmsResources.getPropertyValue("initialCtxID");
	}
	public Rectangle calculateRect(ClientModelController controller, ClientModelController generalMapController) {
		//	currentBBox in the mainMap
		BoundingBox mapBbox = controller.getContext().getGeneral().getBoundingBox();
		//Bbox of the GeneralMap
		BoundingBox generalMapBbox = generalMapController.getContext().getGeneral().getBoundingBox();
		double realWidthOfMap = generalMapBbox.getMaxx().doubleValue() - generalMapBbox.getMinx().doubleValue();
		double realHeightOfMap = generalMapBbox.getMaxy().doubleValue() - generalMapBbox.getMiny().doubleValue();
		// Welche Groesse entspricht ein Pixel in der realen welt?
		double realPixelX = realWidthOfMap / generalMapController.getContext().getGeneral().getWindow().getWidth();
		double realPixelY = realHeightOfMap / generalMapController.getContext().getGeneral().getWindow().getHeight();
		// Position der Rechtecks in der MiniKarte
		double absolutePosX = (mapBbox.getMinx().doubleValue() - generalMapBbox.getMinx().doubleValue()) / realPixelX;
		double absolutePosY = (int) (generalMapBbox.getMaxy().doubleValue() - mapBbox.getMaxy().doubleValue()) / realPixelY;

		double rectHeight = (mapBbox.getMaxy().doubleValue() - mapBbox.getMiny().doubleValue()) / realPixelY;
		double rectWidth = (mapBbox.getMaxx().doubleValue() - mapBbox.getMinx().doubleValue()) / realPixelX;
        if(rectHeight < 1)
            rectHeight = 1;
        if (rectWidth < 1)
            rectWidth = 1;
		return new Rectangle(new BigDecimal(absolutePosX).intValue(), new BigDecimal(absolutePosY).intValue(), new BigDecimal(rectWidth).intValue(), new BigDecimal(rectHeight).intValue());
	}
}
