package org.n52.wmsclientcore;

import java.awt.image.renderable.ParameterBlock;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import javax.media.jai.JAI;
import javax.media.jai.PlanarImage;
import javax.media.jai.RenderedOp;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.log4j.Logger;

import com.sun.media.jai.codec.ImageCodec;
import com.sun.media.jai.codec.ImageEncoder;
import com.sun.media.jai.codec.PNGEncodeParam;
import com.sun.media.jai.codec.SeekableStream;

/**
 * @author Theodor
 *

 */
public class ImageStreamingServlet extends HttpServlet {

    private static Logger logger = Logger.getLogger(ImageStreamingServlet.class.getName());
	
    /**
     * @see javax.servlet.http.HttpServlet#doGet(HttpServletRequest, HttpServletResponse)
     */
    protected void doGet(HttpServletRequest req, HttpServletResponse res)
            throws ServletException, IOException {

        res.setContentType(WmsResources.getPropertyValue("imageFormat"));
        if (req.getParameter("Error") != null) {
            String err = req.getParameter("Error");
            sendError(err, res);
        }

        HttpSession session = req.getSession();
        ClientModelController controller= (ClientModelController) session.getAttribute("controller");

        if (controller.getCollection().isEmpty()) {
            sendError("NoWms", res);
        }
        else if(controller.getVisibleLayers().getLayerCount()==0) 
        	sendError("NoLayer", res);
        else {

            logger.debug("Assembling and encoding map ...");
            try {
                PlanarImage image = controller.getMap();
                logger.debug("Starting map encoding ...");
                PNGEncodeParam ep = PNGEncodeParam.getDefaultEncodeParam(image);
                ep.setInterlacing(false);

                OutputStream out = res.getOutputStream();
                ImageEncoder ie = ImageCodec.createImageEncoder("PNG", out, ep);
                ie.encode(image);
                logger.debug("Request successfully handled");
            } catch (RuntimeException e) {
                logger.debug("Error while assembling/encoding map", e);
            }
        }
    }

    private void sendError(String error, HttpServletResponse response) {
        InputStream errorImage = null;
        if (error.equals("NoLayer")) {
            errorImage = this.getClass().getResourceAsStream(WmsResources.getPropertyValue("errorImageNoLayer"));
        } else if (error.equals("NoWms")) {
            errorImage = this.getClass().getResourceAsStream(WmsResources.getPropertyValue("errorImageNoWms"));
        }

        // Create the ParameterBlock.
        ParameterBlock pb = new ParameterBlock();
        SeekableStream s = SeekableStream.wrapInputStream(errorImage, false);

        // Create the ParameterBlock and add the SeekableStream to it.
        pb.add(s);

        // Create the PNG operation.
        RenderedOp op = JAI.create("PNG", pb);
        PlanarImage image = op.createInstance();

        PNGEncodeParam ep = PNGEncodeParam.getDefaultEncodeParam(image);
        ep.setInterlacing(false);

        try {
            ImageEncoder ie = ImageCodec.createImageEncoder("PNG", response.getOutputStream(), ep);
            ie.encode(image);
        } catch (IOException io) {
            logger.error("Error handling errorImage", io);
        }
    }
}
