/*
 * Created on 31.03.2003
 *
 * To change this generated comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package org.n52.wmsclientcore;

import java.awt.geom.Point2D;
import java.io.IOException;
import java.io.Serializable;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.Logger;
import org.n52.wmsclientcore.capabilities.CapabilitiesFactory;
import org.n52.wmsclientcore.capabilities.model.BoundingBox;
import org.n52.wmsclientcore.capabilities.model.Layer;
import org.n52.wmsclientcore.capabilities.model.WMS_Capabilities;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;


/**
 * @author <a href="mailto:senkler@ifgi.uni-muenster.de">Kristian Senkler</a>
 *
 * A WebMapService is a nice thing.
 */
public class WebMapService implements Serializable {

	private static Logger logger = Logger.getLogger(WebMapService.class.getName());

	
	
	protected String onlineRessource = "";
	protected String title = "";
	protected String version = "";
	protected WMS_Capabilities capabilities;
	protected HashMap <String, Document> sldUrls;

	/**
	 * Constructor
	 */
	public WebMapService() {

	}
	/**
	 * Instantiates a new WebMapService from its url
	 * @param url
	 */
	public WebMapService(String url) throws WmsException {
		this(url, "1.1.0");
	}
	public WebMapService(String url, String initialVersion) throws WmsException {
		this.onlineRessource = url;
		this.init(initialVersion);
		this.version = capabilities.getVersion();
		this.title = getCapabilities().getService().getTitle();
		sldUrls = new HashMap();
	}
	public void addSldUrl(String url) throws IOException, SAXException
	{
        Document document = null;
        try {
            document = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(url);
        }
        catch (ParserConfigurationException e) {
            
        }
		sldUrls.put(url, document);
	}
	public void deleteSldUrl(String url)
	{
		sldUrls.remove(url);
		
	}
	public boolean hasSld(String url)
	{
		return sldUrls.containsKey(url);
	}
	
	
	public String[] getSldUrls()
	{
		String[] urls = new String[sldUrls.size()];
		int urlCounter = sldUrls.size();
		for(Iterator iter = sldUrls.keySet().iterator(); iter.hasNext();)
		{
			String url = (String)iter.next();
			urls[urlCounter-1]=url;
			urlCounter--;
		}	
		return urls;
	}
	
	public Document getSldDocument(String url)
	{
		return (Document)sldUrls.get(url);
	}
	/*************************************************
	 * Basic Methods to identify and describe MWS
	 *
	 *************************************************/
	public WMS_Capabilities getCapabilities() {
		return capabilities;
	}
	/**
	 * Returns a list of all map layers.
	 * @return ArrayList
	 */

	public Layer getRootLayer() {
		return this.capabilities.getCapability().getLayer();
	}
	/**
	 * @return String
	 */
	public String getTitle() {
		return title;
	}

	/**
	 * @return String
	 */
	public String getVersion() {
		return version;
	}

	/**
	 * @param onlineRessource
	 */
	public void setOnlineRessource(String onlineRessource) {
		this.onlineRessource = onlineRessource;
	}

	/**
	 * @return String
	 */
	public String getOnlineRessource() {
		return onlineRessource;
	}

	/**
	 * Checks, if the WMS has a specific MapLayer identified
	 * by 'layerName'
	 * @param layerName
	 * @return boolean
	 */
	public boolean hasMapLayer(String layerName) {
		org.n52.wmsclientcore.capabilities.model.Layer layer = getCapabilities().getCapability().getLayer();
		return searchForLayerName(layer, layerName);
	}
	/*
	 * recursive iterating over Layers...
	 */
	protected boolean searchForLayerName(org.n52.wmsclientcore.capabilities.model.Layer layer, String layerName) {
		boolean ret = false;
		if (layer.getName().equals(layerName)) {
			ret = true;
		} else {

			for (int i = 0; i < layer.getLayerCount(); i++) {
				if (layer.getLayer(i).getName().equals(layerName)) {
					ret = true;
					break;
				}
				if (layer.getLayer(i).getLayer().length < 0)
					ret = searchForLayerName(layer.getLayer(i), layerName);
			}
		}
		return ret;
	}
	/**
	 * Returns the full extend of all MapLayers according to a given SRS.
	 * @param inSRS
	 * @return WmcBoundingBox
	 */
	public WmcBoundingBox getFullExtend(String inSRS) {
		ArrayList list = null;
		WmcBoundingBox resultBox = null;

		try {
			list = getMapLayers();
		} catch (Exception e) {
			e.printStackTrace();
		}

		Point2D ll = null;
		Point2D ur = null;
		for (Iterator i = list.iterator(); i.hasNext();) {
			org.n52.wmsclientcore.capabilities.model.Layer mLayer = (org.n52.wmsclientcore.capabilities.model.Layer) i.next();

			// get bbox with appropriate srs
			BoundingBox[] bbox = mLayer.getBoundingBox();
			WmcBoundingBox wmcBbox = null;
			for (int ii = 0; ii < bbox.length; ii++) {
				if (bbox[ii].getCRS().equals(inSRS)) {
					ll = new Point2D.Double(bbox[ii].getMinx(), bbox[ii].getMiny());
					ur = new Point2D.Double(bbox[ii].getMaxx(), bbox[ii].getMaxy());
					wmcBbox = new WmcBoundingBox(inSRS, ll, ur);
					if (resultBox == null)
						resultBox = wmcBbox;
					else {
						resultBox.union(wmcBbox);
					}
				}
			}

			/*
			            if (ll == null && ur == null) {
			                ll = wmcBbox.getLowerLeftAsDoublePoint();
			                ur = wmcBbox.getUpperRightAsDoublePoint();
			            } else {
			                if (wmcBbox.getLowerLeftAsDoublePoint().getX() < ll.getX()) {
			                    ll.setLocation(wmcBbox.getLowerLeftAsDoublePoint().getX(), ll.getY());
			                }
			                if (wmcBbox.getLowerLeftAsDoublePoint().getY() < ll.getY()) {
			                    ll.setLocation(ll.getX(), wmcBbox.getLowerLeftAsDoublePoint().getY());
			                }
			                if (wmcBbox.getUpperRightAsDoublePoint().getX() > ur.getX()) {
			                    ur.setLocation(wmcBbox.getUpperRightAsDoublePoint().getX(), ur.getY());
			                }
			                if (wmcBbox.getUpperRightAsDoublePoint().getY() > ur.getY()) {
			                    ur.setLocation(ur.getX(), wmcBbox.getUpperRightAsDoublePoint().getY());
			                }
			            }
			 */
		}
		//        resultBox.setBBox(ll, ur);
		return resultBox;
	}

	/**
	 * Returns a list of all map layers.
	 * @return ArrayList
	 */
	public ArrayList getMapLayers() {
		ArrayList retList = new ArrayList();
		this.getRootLayer().fillLayerList(retList);
		return retList;
	}


	/**
	 * Returns a map layer by a given name. If layer is not found
	 * NULL is returned
	 * @param name
	 * @return Layer - map layer with specified name
	 */
	public Layer getMapLayerByName(String name) {
		Iterator itr = this.getMapLayers().iterator();
		while (itr.hasNext()) {
			Layer layer = (Layer) itr.next();
			if (this.tolerantEquals(layer, name))
				return layer;
		}
		return null;
	}
	public boolean layerExists(String name) {
		Iterator itr = this.getMapLayers().iterator();
		while (itr.hasNext()) {
			Layer layer = (Layer) itr.next();
			if (tolerantEquals(layer, name))
				return true;
		}
		return false;
	}
	/*
	 * looks if the specified is named and if the layer is 
	 */
	private boolean tolerantEquals(Layer layer, String name) {
		if (layer.isNamed()) {
			if (layer.getName().equals(name))
				return true;
			else
				return false;
		} else
			return false;
	}

	public void init(String initialVersion) throws WmsException {

		/*
		 * The Request is compliant to 1.1.0
		 */
		try {
			/*if (CapabilitiesCache.getInstance().containsKey(onlineRessource) && CapabilitiesCache.cachingEnabled()) {
				capabilities = (WMS_Capabilities) CapabilitiesCache.getInstance().get(onlineRessource);
				logger.info("getting caps from cache: " + onlineRessource);
			} else {*/

				capabilities = CapabilitiesFactory.createCapabilities(new URL(new org.n52.wmsclientcore.operations.GetCapabilitiesOperation().getGetCapabilitiesRequest(initialVersion, this.onlineRessource).getCompleteRequestURL()));

			
		} catch (Exception e) {
			throw new WmsException("no caps build, request:" +  onlineRessource, "0", e);
		}
		if (CapabilitiesCache.cachingEnabled())
			CapabilitiesCache.set(onlineRessource, capabilities);
		logger.info("getting caps from server: " + onlineRessource);
	}


}