package org.n52.wmsclientcore;

//imports
import java.io.PrintStream;
import java.io.PrintWriter;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;


public class WmsException extends Exception {

    // static attributes...
    public final static String CODE_UNSPECIFIED = "Unspecified";

    // public attributes
    
    
    // private attributes
    private String mCode = CODE_UNSPECIFIED;

    private Exception mRootCause;

    // static methods
    
    
    // constructors
    public WmsException(String msg, String code)
    {
      super(msg);
      this.mCode = code;
    }

    public WmsException(String msg)
    {
      super(msg);
    }

    public WmsException(Exception rootCause)
    {
      this("Exception during service execution", rootCause);
    }

    public WmsException(String msg, Exception rootCause)
    {
      this(msg, CODE_UNSPECIFIED, rootCause);
    }

    public WmsException(String msg, String code, Exception rootCause)
    {
      this(msg, code);
        mRootCause = rootCause;
    }


    // public methods

    public Document toDocument() {
        Document doc = null;
        try {
        	doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
            Element root = doc.createElement("ServiceExceptionReport");
            root.setAttribute("version", "1.1.0");
            doc.appendChild(root);
            Element elem = doc.createElement("ServiceException");
            elem.setAttribute("code", this.mCode);
            elem.setNodeValue(this.getMessage());
            root.appendChild(elem);
        }
        catch(ParserConfigurationException e) {
            //NOTHING HAPPENS
        }
       return doc;
    }

    public String toString() {
        Document doc = toDocument();
        Transformer transformer = null;
        try {
            transformer = TransformerFactory.newInstance().newTransformer();
        } 
        catch (TransformerConfigurationException e1) {
        } 
        catch (TransformerFactoryConfigurationError e1) {
        }
        StreamResult result = new StreamResult();
        DOMSource source = new DOMSource(doc);
        try {
            transformer.transform(source, result);
        } catch (TransformerException e) {
        }
        return doc.toString();

    }

    public String getmCode() {
        return mCode;
    }

    public void setmCode(String mCode) {
        this.mCode = mCode;
    }

    public Exception getRootCause() {
        return mRootCause;
    }

    public void printStackTrace() {
        printStackTrace(System.out);
    }

    public void printStackTrace(PrintWriter out) {
        if (mRootCause != null) {
            out.println("<----------------------- Root cause ----------------------->");
            mRootCause.printStackTrace(out);
        }
        try { super.printStackTrace(out); } catch(Exception e){;}
    }

    public void printStackTrace(PrintStream out) {
        if (mRootCause != null) {
            out.println("<----------------------- Root cause ----------------------->");
            mRootCause.printStackTrace(out);
        }
        try { super.printStackTrace(out); } catch(Exception e){}
    }

    public String getMessage() {
        String msg = "";
        try { msg = super.getMessage();} catch(Exception e) {}
        if (mRootCause != null) {
            msg = msg + " [Root cause: " + mRootCause.getMessage() + "]";
        }
        return msg;
    }
}


