package org.n52.wmsclientcore;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.util.Properties;

import org.apache.commons.collections.map.LRUMap;
import org.apache.log4j.PropertyConfigurator;
import org.exolab.castor.xml.MarshalException;
import org.exolab.castor.xml.ValidationException;
import org.n52.wmsclientcore.context.ViewContext;
import org.n52.wmsclientcore.util.SniffedXMLInputStream;

/**
 * This class includes all configuration, which are necessary for the clientCore.
 * 
 * Logging and configuration will be done by {@link #initProperties()}.
 * @author tfoerste
 *
 */
public class WmsResources {

    private static String wmsClientRessources =      "/resources/wmsclientcore.properties";
    public static String wmsContextRessource =       "/org/n52/wmsclientcore/defaultContext.xml";
    private static String log4JPropertiesRessource = "/resources/log4j.properties";;
    private static Properties wmsClientProperties =   null;

    public static ViewContext getInitialContext() throws WmsException {
        try {
            File file = new File(wmsContextRessource);
            InputStream stream = null;
            if(!file.exists()) {
               stream = WmsResources.class.getResourceAsStream(wmsContextRessource);
            }
            else {
                stream = new FileInputStream(file);
            }
            SniffedXMLInputStream sniffedStream = new SniffedXMLInputStream(stream);
            return (ViewContext) ViewContext.unmarshal(new InputStreamReader(
                    sniffedStream,
                    sniffedStream.getXmlEncoding()));

        } catch (MarshalException e) {
            throw new WmsException(e);
        } catch (ValidationException e) {
            throw new WmsException(e);
        } catch (FileNotFoundException e) {
            throw new WmsException(e);
        } catch (UnsupportedEncodingException e) {
            throw new WmsException(e);
        }
        catch (IOException e) {
            throw new WmsException(e);
        }

    }
    /**
     * Returns a new Logger instance for wmsClient
     * 
     * @param logger
     * @return Logger
     */
    public static void initLogger() {
        Properties props = null;
        PropertiesLoader loader = null;
        try {
            loader = new PropertiesLoader();
            props = loader
                    .getPropertiesByRelativeWorkPath(log4JPropertiesRessource);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        PropertyConfigurator.configure(props);
    }

    /**
     * initiates import properties according to path defined by.
     */
    public static void initProperties() {
        PropertiesLoader loader = new PropertiesLoader();
        initLogger();
        try {
            wmsClientProperties = loader
                    .getPropertiesByRelativeWorkPath(wmsClientRessources);

            //CapabilitiesCache.instance= new
            // LRUMap(Integer.parseInt(getPropertyValue("imageCacheSize")));

            ImageCache.instance = new LRUMap(Integer
                    .parseInt(getPropertyValue("capabilitiesCacheSize")));

            ImageCache.setCachingState(Boolean.valueOf(
                    WmsResources.getPropertyValue("imageCaching"))
                    .booleanValue());
            // CapabilitiesCache is no more used.
            //CapabilitiesCache.setCachingState(Boolean.valueOf(WmsResources.getPropertyValue("capabilitiesCaching")).booleanValue());
            //context = (ViewContext) ViewContext.unmarshal(new FileReader(
            //        new File(wmsContextRessource)));
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    /**
     * Returns a global property value, if it exists. If not, method returns
     * null.
     * 
     * @param key
     * @return String
     */
    public static String getPropertyValue(String key) {
        if(wmsClientProperties == null) {
            initProperties();
        }
        String value = null;
        value = wmsClientProperties.getProperty(key);
        if (value == null || value == "") {
            return null;
        }
        return value;
    }

    public static void setPropertyValue(String key, String value) {
        wmsClientProperties.setProperty(key, value);
    }

    public static void saveProperties() {
        try {
            URL url = WmsResources.class.getResource(wmsClientRessources);
            wmsClientProperties.store(new FileOutputStream(new File(url
                    .getFile())), "");
            checkSystemIntegrity();
        } catch (FileNotFoundException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    /**
     * @return
     */
    public static Properties getWmsClientProperties() {
        return wmsClientProperties;
    }

    /**
     * this Method checks, if caching has to be switched off!
     *  
     */
    private static void checkSystemIntegrity() {
        CapabilitiesCache.setCachingState(Boolean.valueOf(
                WmsResources.getPropertyValue("capabilitiesCaching"))
                .booleanValue());
        ImageCache.setCachingState(Boolean.valueOf(
                WmsResources.getPropertyValue("imageCaching")).booleanValue());

        CapabilitiesCache.instance = new LRUMap(Integer
                .parseInt(getPropertyValue("capabilitiesCacheSize")));

        ImageCache.instance = new LRUMap(Integer
                .parseInt(getPropertyValue("imageCacheSize")));
    }
}