/*
 * Created on 14.11.2003
 */
package org.n52.wmsclientcore.capabilities;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.n52.wmsclientcore.WmsException;
import org.n52.wmsclientcore.capabilities.model.WMS_Capabilities;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;


/**
 * @author Theodor
 */
public class CapabilitiesFactory {
	/**
	 * 
	 * @return an CapabilitiesObject according to its own version
	 */
	public static WMS_Capabilities createCapabilities(URL url) throws WmsException {
	    try{
            return createCapabilities(url.openStream());
        }
        catch (IOException e) {
            new WmsException(e);
        }
        return null;
    }
    
    private static WMS_Capabilities createCapabilities(InputStream stream) throws WmsException {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setNamespaceAware(true);
        Document document = null;
        try {
            DocumentBuilder builder = factory.newDocumentBuilder();
            document = builder.parse(stream);
        }
        catch(IOException e) {
            throw new WmsException(e);
        }
        catch(SAXException e) {
            throw new WmsException(e);
        }
        catch(ParserConfigurationException e) {
            throw new WmsException(e);
        }
        if (document != null) {
            return initCapabilities(document);
        }
        else {
            throw new WmsException("CapabilitiesDocument is null");
        }
    }   
    
    private static WMS_Capabilities initCapabilities(Document doc) throws WmsException
    {
        Element root = doc.getDocumentElement();
        String capVersion = root.getAttribute("version");

        // incl. unofficial version 1.1.3 workaround to get ceoware2.ccrs.nrcan.gc.ca/cubewerx/cubeserv/cubeserv.cgi working 
        // incl. unofficial version 1.1.2 workaround to get mapserv2.esrin.esa.it/cubestor/cubeserv/cubeserv.cgi working 
        if (capVersion.startsWith("1.0")) {
            return new org.n52.wmsclientcore.capabilities.version100.WMT_MS_Capabilities(root);
        } else if (capVersion.equals("1.1.0")) {
            return new org.n52.wmsclientcore.capabilities.version110.WMT_MS_Capabilities(root);}
        else if (capVersion.equals("1.1.1") || capVersion.equals("1.1.2") || capVersion.equals("1.1.3")) {
            return new org.n52.wmsclientcore.capabilities.version111.WMT_MS_Capabilities(root);
        }
        else throw new WmsException("Stated capabilities version "+capVersion+" not supported.");
         
    }
    
    public static WMS_Capabilities createCapabilities(File file) throws WmsException
    {
        try {
            return createCapabilities(new FileInputStream(file));
        }
        catch(FileNotFoundException e) {
            new WmsException(e);
        }
        return null;
    }
	public static void main(String[] args) {
		try {
			//CapabilitiesFactory.createCapabilities(new URL("http://inspire.geodan.nl/cgi-bin/wms/SclMapServer.exe?REQUEST=GETCAPABILITIES&SERVICE=WMS"));
			long before = System.currentTimeMillis();
			WMS_Capabilities caps =
				//				CapabilitiesFactory.createCapabilities(
		//					new URL("http://www.digitalearth.gov/wmt/xml/capabilities_1_0_0.xml"));
	CapabilitiesFactory.createCapabilities(new URL("http://www.digitalearth.gov/wmt/xml/capabilities_1_1_1.xml"));
			long after = System.currentTimeMillis();
			long actual = after - before;
			System.out.println("elapsed Time to retrieve caps and parse it " + actual);

		} catch (MalformedURLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (WmsException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
