/*
 * Created on 25.05.2004
 *
 */
package org.n52.wmsclientcore.operations;

import org.n52.wmsclientcore.capabilities.model.WMS_Capabilities;
import org.n52.wmsclientcore.request.*;


/**
 * @author tfoerste
 *
 */
public class GetFeatureInfoOperation extends WmsOperation {
  
    /**
     * all featureInfoFormats (mime types) supported.
     */
    private String[] formatMimeTypes;
    
	public GetFeatureInfoOperation(WMS_Capabilities caps)
	{
		super(caps);
		this.formatMimeTypes = getFormatsFromCapabilities();
	}
	public GetFeatureInfoRequest getFeatureInfoRequest()
	{
		String version = retrieveVersion();
		if(version.equals("1.1.1"))
			return new GetFeatureInfoRequest111(this.getUrl());
		else if(version.equals("1.1.0"))
			return new GetFeatureInfoRequest110(this.getUrl());
		else if (version. equals("1.0.0"))
			return new GetFeatureInfoRequest100(this.getUrl());
		else return null;
	}
	
	public String getUrl()
	{
		return capabilities.getCapability().getRequest().getGetFeatureInfo().getDCPType(0).getHTTP().getHTTPItem(0).getGet().getOnlineResource().getHref();
	}
	
	/**
	 * checks if this server supports the specified mimeType as
	 * featureInfoFormat.
	 * @param mimeType
	 * @return
	 */
	public boolean supportsFormat(String mimeType) {
	  for (int i = 0; i < this.formatMimeTypes.length; i++) {
	    if (this.formatMimeTypes[i].equalsIgnoreCase(mimeType)) {
	      return true;
	    }
	  }
	  return false;
	}
	
	
	/**
	 * fetches all supported FeatureInfoFormats from Capabilities.
	 * @return String array with supported mime types.
	 */
	public String[] getFormatsFromCapabilities() {
	  return capabilities.getCapability().getRequest().getGetFeatureInfo().getFormat();
	}
	
}
